/*
 ver S_4 на основе P_5/S_3 
 - (S_1) фиксация давления + обнаружение старта 
 - (S_2) c включением передатчика на тесте и после окончания записи
 - (S_3) ПРД вкл на этапе обнаружения старта - плохо - ложный старт (убрано)
 -       добавлена фиксация температуры в максимуме и на финише + напряж.в конце
 -       уменьшен период сохранения результата до старта (1 сек)
 - (S_4) этап теста увеличен с тональной индикацией 4-3-2-1 (40 сек до начала)
 
Включение ВМР280 и RF433 к Digispark
6 контактов 
Р0 = DATA       out  (RF433  FS1000A)    //MISO (SDO) input
P1 = SCK        out  (BMP280)            //CS         out
P2 = MOSI (SDI) out  (BMP280)            //MOSI (SDI) out
P3 = CS         out  (BMP280)            //SCK        out
P4 = MISO       out  (BMP280)            //GND        out
P5 = NC                                  //VCC (RESET) - 300 ом к VCC
  
высотомер: пишет в EEPROM отсчеты давления
- если при включении неисправен BMP280 - частые мигания
- иначе - до начала фиксации данных три двойные вспышки
- определение старта по изменению высоты (давления) на  Hd от начальной
(до старта одиночные мигания 1/8   2 сек)
- после обнаружения старта светодиод помаргивает (CS совпадает с светодиодом)
- после заполнения памяти светодиод гаснет

В начальные ячейки добавлена запись:
температуры с ВМР280; напряжения питания; начального давления: 
000 температура (+20 *100)
002 напряжение VCC (в мВ)
004 начальное значение давления (в Па - 80000) (медиана из 3х замеров)
006 температура в максимуме (-5м) (+20 *100)
008 значение адреса замера температуры в максимуме
010 температура после заполнения памяти (+20 *100)
012 напряжение VCC после заполнения памяти (в мВ)
 */

#define _cs    3  //1  //0
#define _mosi  2  //2  //1
#define _miso  4  //0  //2
#define _sck   1  //3

//#define _gnd   4   //GND BMP280
#define led    1   //светодиод тут
#define rf     0   //вход DATA передатчика FS1000A

//#include <DigiUSB.h>
#include <EEPROM.h>
//#include <DigiCDC.h>

#define BMP280_REGISTER_CHIPID  0xD0
#define BMP280_REGISTER_CONTROL 0xF4
#define BMP280_REGISTER_PRESSUREDATA       0xF7
#define BMP280_REGISTER_TEMPDATA           0xFA
#define BMP280_REGISTER_DIG_T1  0x88
#define BMP280_REGISTER_DIG_T2  0x8A
#define BMP280_REGISTER_DIG_T3  0x8C

#define BMP280_REGISTER_DIG_P1  0x8E
#define BMP280_REGISTER_DIG_P2  0x90
#define BMP280_REGISTER_DIG_P3  0x92
#define BMP280_REGISTER_DIG_P4  0x94
#define BMP280_REGISTER_DIG_P5  0x96
#define BMP280_REGISTER_DIG_P6  0x98
#define BMP280_REGISTER_DIG_P7  0x9A
#define BMP280_REGISTER_DIG_P8  0x9C
#define BMP280_REGISTER_DIG_P9  0x9E

/*=========================================================================
    CALIBRATION DATA
    -----------------------------------------------------------------------*/
    typedef struct
    {
      uint16_t dig_T1;
      int16_t  dig_T2;
      int16_t  dig_T3;

      uint16_t dig_P1;
      int16_t  dig_P2;
      int16_t  dig_P3;
      int16_t  dig_P4;
      int16_t  dig_P5;
      int16_t  dig_P6;
      int16_t  dig_P7;
      int16_t  dig_P8;
      int16_t  dig_P9;
    } bmp280_calib_data;
/*=========================================================================*/

    int32_t t_fine;

    bmp280_calib_data _bmp280_calib;
//unsigned int voltage;
#define N0 14 //было 22   //начало области EEPROM для отсчетов высоты
#define Alt0  1025.25   //давление на высоте 0
#define MasMax 10       //размер массива в ОЗУ (по 2 байта) для записи отсчетов до старта
int addr = N0;  //пишем отсчеты не с начала EEPROM
//int itmp = 0;
#define tPress 100   //период опроса датчика
#define maxAddr 512 //объем EEPROM ATtiny85 =512
int maxAlt = 0;      //значение давления при максимальной высоты
#define maxN 5  //3  //5       //число подряд идущих измерений высот меньших макс для опр.снижения
#define minD 60  //10   //100  //минимальное снижение (повышение давления) от максимума высоты (минимума давления)
     //для определения снижения (на более чем 5м) 12 отсчетов на 1м
int N = 0;       //подсчет числа повторов снижения высоты (до maxN)
//int ignitON = 0;   //признак "включен ли запал" (0 - не включен)
byte aLow;
byte aHigh;
unsigned int valAlt;
long int valPress;

//long voltage;
//-------------
int Nbuf = 0;          //указатель ОЗУ (до старта отсчеты пишем в ОЗУ)
byte Mas[MasMax*2];    //сам массив в ОЗУ для хранения отсчетов до взлета
unsigned int Hmax = 0;   //тут отслеживаем высоту (давление) при определении взлета 
unsigned int H0 = 0;     //тут высота (давление) на старте 
//int Kpovt = 0;         //число повторов увеличения высоты при взлете
//#define Hd 10 //50          //константа для определения взлета (5м)
#define Hd 13 //60          //константа для определения взлета - падение давления (примерно 1м) 12 отсчетов на 1м (5м)
#define Kmax 3 //5         //константа число подряд изм.высоты больше макс. для опр.взлета

uint8_t BMP280_spixfer(uint8_t x) {
  // software spi
  uint8_t reply = 0;
  for (int i=7; i>=0; i--) {
    reply <<= 1;
    digitalWrite(_sck, LOW);
    digitalWrite(_mosi, x & (1<<i));
    digitalWrite(_sck, HIGH);
    if (digitalRead(_miso))
      reply |= 1;
  }
  return reply;
}

void write8(byte reg, byte value)
{
    digitalWrite(_cs, LOW);
    BMP280_spixfer(reg & ~0x80); // write, bit 7 low
    BMP280_spixfer(value);
    digitalWrite(_cs, HIGH);
}

uint8_t read8(byte reg)
{
  uint8_t value;

    digitalWrite(_cs, LOW);
    BMP280_spixfer(reg | 0x80); // read, bit 7 high
    value = BMP280_spixfer(0);
    digitalWrite(_cs, HIGH);
  return value;
}

uint16_t read16(byte reg)
{
  uint16_t value;

   digitalWrite(_cs, LOW);
    BMP280_spixfer(reg | 0x80); // read, bit 7 high
    value = (BMP280_spixfer(0) << 8) | BMP280_spixfer(0);
    digitalWrite(_cs, HIGH);

  return value;
}

uint16_t read16_LE(byte reg) {
  uint16_t temp = read16(reg);
  return (temp >> 8) | (temp << 8);

}
/*
int16_t readS16(byte reg)
{
  return (int16_t)read16(reg);

}
*/
int16_t readS16_LE(byte reg)
{
  return (int16_t)read16_LE(reg);

}

uint32_t read24(byte reg)
{
  uint32_t value;

   digitalWrite(_cs, LOW);
    BMP280_spixfer(reg | 0x80); // read, bit 7 high
    
    value = BMP280_spixfer(0);
    value <<= 8;
    value |= BMP280_spixfer(0);
    value <<= 8;
    value |= BMP280_spixfer(0);

    digitalWrite(_cs, HIGH);

  return value;
}


void BMP280_readCoefficients(void)
{
    _bmp280_calib.dig_T1 = read16_LE(BMP280_REGISTER_DIG_T1);
    _bmp280_calib.dig_T2 = readS16_LE(BMP280_REGISTER_DIG_T2);
    _bmp280_calib.dig_T3 = readS16_LE(BMP280_REGISTER_DIG_T3);

    _bmp280_calib.dig_P1 = read16_LE(BMP280_REGISTER_DIG_P1);
    _bmp280_calib.dig_P2 = readS16_LE(BMP280_REGISTER_DIG_P2);
    _bmp280_calib.dig_P3 = readS16_LE(BMP280_REGISTER_DIG_P3);
    _bmp280_calib.dig_P4 = readS16_LE(BMP280_REGISTER_DIG_P4);
    _bmp280_calib.dig_P5 = readS16_LE(BMP280_REGISTER_DIG_P5);
    _bmp280_calib.dig_P6 = readS16_LE(BMP280_REGISTER_DIG_P6);
    _bmp280_calib.dig_P7 = readS16_LE(BMP280_REGISTER_DIG_P7);
    _bmp280_calib.dig_P8 = readS16_LE(BMP280_REGISTER_DIG_P8);
    _bmp280_calib.dig_P9 = readS16_LE(BMP280_REGISTER_DIG_P9);
}

bool BMP280_begin() {
    digitalWrite(_cs, HIGH);
    pinMode(_cs, OUTPUT);

      // software SPI
      pinMode(_sck, OUTPUT);
      pinMode(_mosi, OUTPUT);
      pinMode(_miso, INPUT);

  if (read8(BMP280_REGISTER_CHIPID) != 0x58)
    return false;

  BMP280_readCoefficients();
  write8(BMP280_REGISTER_CONTROL, 0x3F);
  return true;
}

//температура в градусах цельсия * 100 (пример: 1035 -> +10.35C)
long BMP280_readTemperature(void)
{
  int32_t var1, var2;

  int32_t adc_T = read24(BMP280_REGISTER_TEMPDATA);
  adc_T >>= 4;

  var1  = ((((adc_T>>3) - ((int32_t)_bmp280_calib.dig_T1 <<1))) *
     ((int32_t)_bmp280_calib.dig_T2)) >> 11;

  var2  = (((((adc_T>>4) - ((int32_t)_bmp280_calib.dig_T1)) *
       ((adc_T>>4) - ((int32_t)_bmp280_calib.dig_T1))) >> 12) *
     ((int32_t)_bmp280_calib.dig_T3)) >> 14;

  t_fine = var1 + var2;

  long T  = (t_fine * 5 + 128) >> 8;
  return T;
  //в исходнике: return T/100; тип float
}

//давление в Па Q24.8 Пример: 24674867 -> 24674867/256 = 96386.2 Pa
unsigned long BMP280_readPressure(void) {
  int64_t var1, var2, p;

  // Must be done first to get the t_fine variable set up
  BMP280_readTemperature();

  int32_t adc_P = read24(BMP280_REGISTER_PRESSUREDATA);
  adc_P >>= 4;

  var1 = ((int64_t)t_fine) - 128000;
  var2 = var1 * var1 * (int64_t)_bmp280_calib.dig_P6;
  var2 = var2 + ((var1*(int64_t)_bmp280_calib.dig_P5)<<17);
  var2 = var2 + (((int64_t)_bmp280_calib.dig_P4)<<35);
  var1 = ((var1 * var1 * (int64_t)_bmp280_calib.dig_P3)>>8) +
    ((var1 * (int64_t)_bmp280_calib.dig_P2)<<12);
  var1 = (((((int64_t)1)<<47)+var1))*((int64_t)_bmp280_calib.dig_P1)>>33;

  if (var1 == 0) {
    return 0;  // avoid exception caused by division by zero
  }
  p = 1048576 - adc_P;
  p = (((p<<31) - var2)*3125) / var1;
  var1 = (((int64_t)_bmp280_calib.dig_P9) * (p>>13) * (p>>13)) >> 25;
  var2 = (((int64_t)_bmp280_calib.dig_P8) * p) >> 19;

  p = ((p + var1 + var2) >> 8) + (((int64_t)_bmp280_calib.dig_P7)<<4);
  return p;
//    return (float)p/256;

}

/*
//расчет высоты Высота = 44330 (1-(P/Po)в степени 1/5,255)
//см в https://iarduino.ru/lib/datasheet%20bmp180.pdf  в конце
float BMP280_readAltitude(float seaLevelhPa) {
  float altitude;

  float pressure = BMP280_readPressure()/25600; // in Si units for Pascal
//  pressure /= 100;

  altitude = 44330 * (1.0 - pow(pressure / seaLevelhPa, 0.1903));

  return altitude;
}
*/

long readVcc() {
  // Read 1.1V reference against AVcc
  // set the reference to Vcc and the measurement to the internal 1.1V reference
  
  #if defined(__AVR_ATmega32U4__) || defined(__AVR_ATmega1280__) || defined(__AVR_ATmega2560__)
    ADMUX = _BV(REFS0) | _BV(MUX4) | _BV(MUX3) | _BV(MUX2) | _BV(MUX1);
  #elif defined (__AVR_ATtiny24__) || defined(__AVR_ATtiny44__) || defined(__AVR_ATtiny84__)
    ADMUX = _BV(MUX5) | _BV(MUX0);
  #elif defined (__AVR_ATtiny25__) || defined(__AVR_ATtiny45__) || defined(__AVR_ATtiny85__)
    ADMUX = _BV(MUX3) | _BV(MUX2);
  #else
    ADMUX = _BV(REFS0) | _BV(MUX3) | _BV(MUX2) | _BV(MUX1);
  #endif  
 
  delay(2); // Wait for Vref to settle
  ADCSRA |= _BV(ADSC); // Start conversion
  while (bit_is_set(ADCSRA,ADSC)); // measuring
 
  uint8_t low  = ADCL; // must read ADCL first - it then locks ADCH  
  uint8_t high = ADCH; // unlocks both
 
  long result = (high<<8) | low;
 
  result = 1125300L / result; // Calculate Vcc (in mV); 1125300 = 1.1*1023*1000
  return result; // Vcc in millivolts
  
}

//--------------- подпрограммы ---------------------------
//перезапись массива из ОЗУ в EEPROM после взлета
void MasToEE(unsigned int tAlt) {
  //запись в EEPROM штатного замера высоты
//  valAlt = bme.readAltitude(Alt0) * 10; // -100м
//    Serial.print(valAlt);  //bme.readAltitude(1013.25)); // this should be adjusted to your local forcase
  EEPROM.write(addr, lowByte(tAlt));
  EEPROM.write(addr+1, highByte(tAlt));
//  addr ++;  addr ++;   //к следующему адресу
  //если не весь массив из ОЗУ переписан в EEPROM - потихоньку переписываем
  if (Nbuf >= 0) {
    //еще не весь массив перенесен в EEPROM
    EEPROM.write(Nbuf + N0, Mas[Nbuf]);   //одна 2х байтовая ячейка
    EEPROM.write(Nbuf + N0 + 1, Mas[Nbuf+1]);
    Nbuf = Nbuf - 2;  //смещаем указатель массива
//    EEPROM.write(Nbuf + N0, Mas[Nbuf]);   //еще одна ячейка
//    EEPROM.write(Nbuf + N0 + 1, Mas[Nbuf+1]);
//    Nbuf --; Nbuf --;  //смещаем указатель массива
  }
}

//запись отсчетов в массив в ОЗУ до старта
void AltToMas(unsigned int tAlt) {
  Mas[Nbuf] = lowByte(tAlt);
  Mas[Nbuf+1] = highByte(tAlt);
  Nbuf = Nbuf + 2;
  if (Nbuf > MasMax * 2) {
    //весь массив заполнен, надо сдвигать
    for (Nbuf = 2; Nbuf <= MasMax*2; Nbuf ++) {
       Mas[Nbuf-2] = Mas[Nbuf];
    }
    Nbuf = MasMax*2; //ставим на последнюю ячейку
  }  
}
//--------------------------------------------------------

//считываем давление с датчика
//скорректированное давление с привязкой к начальному отсчету
unsigned int readPressure(void)
{
   unsigned long tmp_P = BMP280_readPressure();  //давление в Паскалях (101000 Pa) *256
   return tmp_P/256 - 80000; //чтоб в два байта уложилось   
}

//включить меандр 1000Гц на выводе _pin импульсов числом _nn (это число мс)
void ToneOn(int _pin, int _nn) 
{
  for (int i = 0; i < _nn; i ++) {
       digitalWrite(_pin,HIGH);
       delayMicroseconds(500); 
       digitalWrite(_pin,LOW);
       delayMicroseconds(500); 
  }
}

void setup() {
  addr = N0;      //пишем отсчеты не с начала EEPROM
//  itmp = 0;
//  maxAlt = 0;     //значение максимальной высоты
//  N = 0;    //подсчет числа повторов снижения высоты (до maxN)
//  ignitON = 0;    //признак "включен ли запал" (0 - не включен)
  Nbuf = 0;       //указатель ОЗУ (до старта отсчеты пишем в ОЗУ)
  Hmax = 0;       //тут отслеживаем высоту при определении взлета 
  H0 = 0;         //тут высота на старте 
//  Kpovt = 0;      //число повторов увеличения высоты при взлете
  
  // initialize the digital pin as an output.
  pinMode(led, OUTPUT);     
  pinMode(rf, OUTPUT);     
  digitalWrite(rf, LOW); //выкл ПРД
  
  if (!BMP280_begin()) {  //если ошибка BMP280 - частые мигания
 //       digitalWrite(led, HIGH); //экономим память - просто включим
 //       while (1);

    while (1) {
        digitalWrite(led, HIGH);
        delay(200);  
        digitalWrite(led, LOW); 
        delay(200);               
        ToneOn(rf,200);     //вкл ПРД при ошибке BMP280        
        }
       
    }

  //пауза перед началом обнаружения старта - 4-3-2-1-ые вспышки

  //10 сек = 4 раз по 4 вспышки
  for (int itmp = 0; itmp < 4; itmp++) {  //отсчет 40-ые секунды
     digitalWrite(led, HIGH);   // turn the LED on (HIGH is the voltage level)
     ToneOn(rf,200);            //вкл ПРД         
     digitalWrite(led, LOW);    // turn the LED off by making the voltage LOW
     delay(200);
     digitalWrite(led, HIGH);   // turn the LED on (HIGH is the voltage level)
     ToneOn(rf,200);            //вкл ПРД        
     digitalWrite(led, LOW);    // turn the LED off by making the voltage LOW
     delay(200);
     digitalWrite(led, HIGH);   // turn the LED on (HIGH is the voltage level)
     ToneOn(rf,200);            //вкл ПРД        
     digitalWrite(led, LOW);    // turn the LED off by making the voltage LOW
     delay(200);
     digitalWrite(led, HIGH);   // turn the LED on (HIGH is the voltage level)
     ToneOn(rf,200);            //вкл ПРД        
     digitalWrite(led, LOW);    // turn the LED off by making the voltage LOW
     delay(1100);
  }    
  //10 сек = 5 раз по 3 вспышки
  for (int itmp = 0; itmp < 5; itmp++) {  //отсчет 30-ые секунды
     digitalWrite(led, HIGH);   // turn the LED on (HIGH is the voltage level)
     ToneOn(rf,200);            //вкл ПРД         
     digitalWrite(led, LOW);    // turn the LED off by making the voltage LOW
     delay(200);
     digitalWrite(led, HIGH);   // turn the LED on (HIGH is the voltage level)
     ToneOn(rf,200);            //вкл ПРД        
     digitalWrite(led, LOW);    // turn the LED off by making the voltage LOW
     delay(200);
     digitalWrite(led, HIGH);   // turn the LED on (HIGH is the voltage level)
     ToneOn(rf,200);            //вкл ПРД        
     digitalWrite(led, LOW);    // turn the LED off by making the voltage LOW
     delay(1000);
  }    
  //10 сек = 6 раз по 2 вспышки
  for (int itmp = 0; itmp < 6; itmp++) {  //отсчет 20-ые секунды
     digitalWrite(led, HIGH);   // turn the LED on (HIGH is the voltage level)
     ToneOn(rf,200);            //вкл ПРД         
     digitalWrite(led, LOW);    // turn the LED off by making the voltage LOW
     delay(200);
     digitalWrite(led, HIGH);   // turn the LED on (HIGH is the voltage level)
     ToneOn(rf,200);            //вкл ПРД        
     digitalWrite(led, LOW);    // turn the LED off by making the voltage LOW
     delay(1066);
  }    
  //9 сек = 7 раз по 1 вспышке
  for (int itmp = 0; itmp < 7; itmp++) {  //отсчет 10-ые секунды
     digitalWrite(led, HIGH);   // turn the LED on (HIGH is the voltage level)
     ToneOn(rf,200);            //вкл ПРД         
     digitalWrite(led, LOW);    // turn the LED off by making the voltage LOW
     delay(1100);
  }  
  //2 сек - одна длинная  
     digitalWrite(led, HIGH);   // turn the LED on (HIGH is the voltage level)
     ToneOn(rf,800);            //вкл ПРД         
     digitalWrite(led, LOW);    // turn the LED off by making the voltage LOW
     delay(1200);

     digitalWrite(led, HIGH);   // turn the LED on (HIGH is the voltage level)  
   
//=========== пишем параметры на старте ===========
//"оживим датчик"
/*
  valAlt = BMP280_readTemperature(); //температура с BMP280    000
  long int valPress = BMP280_readPressure();  //давление в Паскалях (101000 Pa)
  valAlt = BMP280_readAltitude(1013.25) * 10; //analogRead(0) / 4;
  
  delay(100);
  */
//снова измерим и запишем

  valAlt = BMP280_readTemperature() +2000; //температура с BMP280 на 20 градусов выше и в 100 раз больше
  EEPROM.write(0, lowByte(valAlt));
  EEPROM.write(1, highByte(valAlt));
/*
  valAlt = BMP280_readPressure()/10; //давление с BMP280 в 10 раз меньше
  EEPROM.write(2, lowByte(valAlt));
  EEPROM.write(3, highByte(valAlt));
*/  
  valAlt = readVcc();                //питание устройства VCC(5V)    002
  EEPROM.write(2, lowByte(valAlt));
  EEPROM.write(3, highByte(valAlt));
  
  //определение исходного давления : три отсчета, берем среднее
//Hmax = BMP280_readTemperature();
Hmax = readPressure();  //на самом деле - давление
delay(300);
H0 = readPressure();  //на самом деле - давление
if (H0 >= Hmax) {
  valAlt = H0;
  H0 = Hmax;
  Hmax = valAlt;
}
// тут имеем H0 <= Hmax
delay(300);
valAlt = readPressure();  //на самом деле - давление
if (valAlt < Hmax) {
  if (valAlt <= H0) Hmax = H0;
  else Hmax = valAlt;
}
  H0 = Hmax;      //высота (давление) перед стартом
  maxAlt = Hmax;  //тут храним минимум давления (макс высота)
  
  EEPROM.write(4, lowByte(H0));   //запишем 004
  EEPROM.write(5, highByte(H0));

//int ii = 0;
  //пишем отсчеты в ОЗУ до обнаружения старта
 do {
    long int yTek = millis();
    valAlt = readPressure();
    AltToMas(valAlt);  //пишем отсчет в ОЗУ

     //контрольное мигание вспышки 1/8   2 сек
     
      int yTmp = highByte(yTek) & B00000111;
      if (yTmp == 0)  {   
          digitalWrite(led, HIGH);   // turn the LED on (HIGH is the voltage level)
//этот код в версии S_3 вызывает ложное обнаружение старта
//ToneOn(rf,50);             //вкл ПРД   на 50мс     
          }
      else {
          digitalWrite(led, LOW);    // turn the LED off by making the voltage LOW
          } 
    
     if (valAlt <= Hmax) Hmax = valAlt;  //с высотой давление падает, поэтому ищем понижение
    
   while ((yTek + tPress) > millis()) {}  //пауза перед след.отсчетом
//   ii ++;
//  } while(ii < 20);   //пока высота не увеличилась на Hd 
} while(Hmax >= (H0 - Hd));   //пока давление не упало на Hd 
  //летим...  идем в основной цикл регистрации
  Nbuf = (MasMax-1)*2;           //подготовка к перезаписи ОЗУ в EEPROM
  addr = MasMax*2 + N0;  //адрес EEPROM для фиксации высоты

  //гасим светодиод - признак "летим"
//          digitalWrite(led, LOW);    // turn the LED off by making the voltage LOW

}

void loop() {
  long int tTek = millis();
  
  //контрольное мигание 1/8   2 сек
  /*
  int tTmp = highByte(tTek) & B00000111;
  if (tTmp == 0)  {   
    digitalWrite(led, HIGH);   // turn the LED on (HIGH is the voltage level)
  }
  else {
    digitalWrite(led, LOW);    // turn the LED off by making the voltage LOW
  }    
  */
//Высота  (давление)
  valAlt = readPressure();
  MasToEE(valAlt);  //запись в EEPROM  
  
/*
  //определяем момент включения "запала системы спасения"
  if (ignitON == 0) {  //только если запал еще не включали 
  */
  if (valAlt < maxAlt) {  //"копим" минимум давления - идет "взлет"
      maxAlt = valAlt;
    N = 0;
    }
  else {  //возможно, пошло падение
      N ++;  //считаем число повторов "понижения" (повышения давления)
    if (N > maxN) {  //число подряд "понижений" превысило порог - можно сказать - "падаем"
        //проверим абсолютное значение "падения" к максимуму
        if ((valAlt - maxAlt) > minD) { //тут замерим температуру и запомним отсчет
          //запишем данные точки
          valAlt = BMP280_readTemperature() +2000; //температура с BMP280 на 20 градусов выше и в 100 раз больше
          EEPROM.write(6, lowByte(valAlt));
          EEPROM.write(7, highByte(valAlt));
          //запишем в EEPROM момент включения "запала" (addr-1)
          EEPROM.write(8, lowByte(addr));
          EEPROM.write(9, highByte(addr));
          //включим "запал"
//          ignitON = 1; //признак "Запал включен"
         }
       }
    }
//  }

  addr = addr + 2;
  if (addr >= maxAddr)  {
 //   addr = 0;  //циклическая перезапись (!?)
 // запись завершена - фиксируем конечные данные
  valAlt = BMP280_readTemperature() +2000; //температура с BMP280 на 20 градусов выше и в 100 раз больше
  EEPROM.write(10, lowByte(valAlt));
  EEPROM.write(11, highByte(valAlt));
  valAlt = readVcc();                //питание устройства VCC(5V)    002
  EEPROM.write(12, lowByte(valAlt));
  EEPROM.write(13, highByte(valAlt));
//память заполнена - светодиод  ГАСИМ
     digitalWrite(led, LOW);
     //включаем передатчик
    while (1) {
        delay(1000);  
        ToneOn(rf,200);     //вкл ПРД для системы обнаружения        
        }
  }
    
  while ((tTek + tPress) > millis()) {} 
}
